"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObjectNode = exports.Case = exports.Switch = void 0;
const AbstractNode_1 = require("./AbstractNode");
const Registries_1 = require("../Registries");
exports.Switch = Symbol('switch');
exports.Case = Symbol('case');
/**
 * Object node containing fields with different types.
 * Has the ability to filter fields based on a switch field.
 */
class ObjectNode extends AbstractNode_1.AbstractNode {
    /**
     * @param fields children containing the optional switch and case
     * @param mods optional node modifiers
     */
    constructor(fields, mods) {
        super({
            default: () => ({}),
            ...mods
        });
        this.collapse = mods === null || mods === void 0 ? void 0 : mods.collapse;
        this.category = mods === null || mods === void 0 ? void 0 : mods.category;
        const { [exports.Switch]: _switch, [exports.Case]: _case, ..._fields } = fields;
        this.fields = _fields;
        this.cases = _case !== null && _case !== void 0 ? _case : {};
        this.filter = _switch;
    }
    transform(path, value, view) {
        if (value === undefined)
            return undefined;
        const activeFields = this.getActiveFields(path, view.model);
        let res = {};
        Object.keys(activeFields).forEach(f => {
            return res[f] = activeFields[f].transform(path.push(f), value[f], view);
        });
        return this.transformMod(res);
    }
    render(path, value, view, options) {
        if (this.collapse || (options === null || options === void 0 ? void 0 : options.collapse)) {
            if (value === undefined) {
                const id = view.registerClick(() => view.model.set(path, this.default()));
                return `<div class="node object-node">
          <div class="node-header">
            <label class="collapse closed" data-id="${id}">${Registries_1.locale(path)}</label>
          </div>
        </div>`;
            }
            else {
                const id = view.registerClick(() => view.model.set(path, undefined));
                return `<div class="node object-node"${this.category ? `data-category="${this.category}"` : ''}>
          <div class="node-header">
            <label class="collapse open" data-id="${id}">${Registries_1.locale(path)}</label>
          </div>
          <div class="node-body">
            ${this.renderFields(path, value, view)}
          </div>
        </div>`;
            }
        }
        else {
            return `<div class="node object-node"${this.category ? `data-category="${this.category}"` : ''}>
        ${(options === null || options === void 0 ? void 0 : options.hideLabel) ? `
          ${(options === null || options === void 0 ? void 0 : options.removeId) ? `<div class="node-header">
            <button class="remove" data-id="${options === null || options === void 0 ? void 0 : options.removeId}">${options === null || options === void 0 ? void 0 : options.removeLabel}</button>
          </div>` : ``}
        ` : `
          <div class="node-header">
            ${(options === null || options === void 0 ? void 0 : options.removeId) ? `<button class="remove" data-id="${options === null || options === void 0 ? void 0 : options.removeId}"></button>` : ``}
            <label>${Registries_1.locale(path)}</label>
          </div>`}
        <div class="node-body">
          ${this.renderFields(path, value, view)}
        </div>
      </div>`;
        }
    }
    renderFields(path, value, view) {
        value = value !== null && value !== void 0 ? value : {};
        const activeFields = this.getActiveFields(path, view.model);
        return Object.keys(activeFields).map(f => {
            if (!activeFields[f].enabled(path.push(f), view.model))
                return '';
            return activeFields[f].render(path.push(f), value[f], view);
        }).join('');
    }
    getActiveFields(path, model) {
        if (this.filter === undefined)
            return this.fields;
        const switchValue = this.filter(path.withModel(model));
        const activeCase = this.cases[switchValue];
        return { ...this.fields, ...activeCase };
    }
    validate(path, value, errors) {
        if (value === null || typeof value !== 'object') {
            return errors.add(path, 'error.expected_object');
        }
        const activeFields = this.getActiveFields(path, path.getModel());
        const activeKeys = Object.keys(activeFields);
        let allValid = true;
        Object.keys(value).forEach(k => {
            if (!activeKeys.includes(k)) {
                if (this.filter) {
                    const switchValue = this.filter(path);
                    errors.add(path.push(k), 'error.invalid_filtered_key', k, switchValue);
                }
                else {
                    errors.add(path.push(k), 'error.invalid_key', k);
                }
                allValid = false;
            }
            else if (!activeFields[k].validate(path.push(k), value[k], errors)) {
                allValid = false;
            }
        });
        return allValid;
    }
}
exports.ObjectNode = ObjectNode;
