"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TreeView = exports.getId = void 0;
const Path_1 = require("../model/Path");
const AbstractView_1 = require("./AbstractView");
const registryIdLength = 12;
const dec2hex = (dec) => ('0' + dec.toString(16)).substr(-2);
/**
 * Helper function to generate a random ID
 */
function getId() {
    var arr = new Uint8Array((registryIdLength || 40) / 2);
    window.crypto.getRandomValues(arr);
    return Array.from(arr, dec2hex).join('');
}
exports.getId = getId;
/**
 * DOM representation view of the model.
 */
class TreeView extends AbstractView_1.AbstractView {
    /**
     * @param model data model this view represents and listens to
     * @param target DOM element to render the view
     */
    constructor(model, target) {
        super(model, target);
        this.registry = {};
    }
    /**
     * Registers a callback and gives an ID
     * @param callback function that is called when the element is mounted
     * @returns the ID that should be applied to the data-id attribute
     */
    register(callback) {
        const id = getId();
        this.registry[id] = callback;
        return id;
    }
    /**
     * Registers an event and gives an ID
     * @param type event type
     * @param callback function that is called when the event is fired
     * @returns the ID that should be applied to the data-id attribute
     */
    registerEvent(type, callback) {
        return this.register(el => {
            el.addEventListener(type, evt => {
                callback(el);
                evt.stopPropagation();
            });
        });
    }
    /**
     * Registers a change event and gives an ID
     * @param callback function that is called when the event is fired
     * @returns the ID that should be applied to the data-id attribute
     */
    registerChange(callback) {
        return this.registerEvent('change', callback);
    }
    /**
     * Registers a click event and gives an ID
     * @param callback function that is called when the event is fired
     * @returns the ID that should be applied to the data-id attribute
     */
    registerClick(callback) {
        return this.registerEvent('click', callback);
    }
    /**
     * @override
     */
    render() {
        this.target.innerHTML = this.model.schema.render(new Path_1.Path(), this.model.data, this, { hideLabel: true });
        for (const id in this.registry) {
            const element = this.target.querySelector(`[data-id="${id}"]`);
            if (element !== null)
                this.registry[id](element);
        }
        this.registry = {};
    }
}
exports.TreeView = TreeView;
